@@ -38,8 +38,7 @@
 #endif
 #include <assert.h>
 #include "common.h"
-
-INTERNAL int code_128(struct zint_symbol *symbol, unsigned char source[], int length);
+#include "code128.h"
 
 #define uchar unsigned char
 
@@ -65,23 +64,6 @@ INTERNAL int code_128(struct zint_symbol *symbol, unsigned char source[], int le
 #define aCodeC (uchar)(134)
 #define aShift (uchar)(135)
 
-static const char *C128Table[107] = {
-    /* Code 128 character encodation - Table 1 */
-    "212222", "222122", "222221", "121223", "121322", "131222", "122213",
-    "122312", "132212", "221213", "221312", "231212", "112232", "122132", "122231", "113222",
-    "123122", "123221", "223211", "221132", "221231", "213212", "223112", "312131", "311222",
-    "321122", "321221", "312212", "322112", "322211", "212123", "212321", "232121", "111323",
-    "131123", "131321", "112313", "132113", "132311", "211313", "231113", "231311", "112133",
-    "112331", "132131", "113123", "113321", "133121", "313121", "211331", "231131", "213113",
-    "213311", "213131", "311123", "311321", "331121", "312113", "312311", "332111", "314111",
-    "221411", "431111", "111224", "111422", "121124", "121421", "141122", "141221", "112214",
-    "112412", "122114", "122411", "142112", "142211", "241211", "221114", "413111", "241112",
-    "134111", "111242", "121142", "121241", "114212", "124112", "124211", "411212", "421112",
-    "421211", "212141", "214121", "412121", "111143", "111341", "131141", "114113", "114311",
-    "411113", "411311", "113141", "114131", "311141", "411131", "211412", "211214", "211232",
-    "2331112"
-};
-
 /* Code F Analysing-Chart */
 typedef struct sCharacterSetTable
 {
@@ -601,7 +583,7 @@ static void SumASCII(uchar **ppOutPos, int Sum, int CharacterSet)
 
 /* Main function called by zint framework
  */
-INTERNAL int codablock(struct zint_symbol *symbol, unsigned char source[], int length) {
+INTERNAL int codablockf(struct zint_symbol *symbol, unsigned char source[], int length) {
     int charCur, dataLength;
     int error_number;
     int rows, columns, useColumns;
@@ -613,7 +595,6 @@ INTERNAL int codablock(struct zint_symbol *symbol, unsigned char source[], int l
     int emptyColumns;
     char dest[1000];
     int r, c;
-    float min_row_height = 0.0f;
 #ifdef _MSC_VER
     CharacterSetTable *T;
     unsigned char *data;
@@ -627,20 +608,20 @@ INTERNAL int codablock(struct zint_symbol *symbol, unsigned char source[], int l
     /* option1: rows <= 0: automatic, 1..44 */
     rows = symbol->option_1;
     if (rows == 1) {
-        error_number = code_128(symbol, source, length); /* Only returns errors, not warnings */
+        error_number = code128(symbol, source, length); /* Only returns errors, not warnings */
         if (error_number < ZINT_ERROR) {
             symbol->output_options |= BARCODE_BIND;
             if (symbol->border_width == 0) { /* Allow override if non-zero */
                 symbol->border_width = 1; /* AIM ISS-X-24 Section 4.6.1 b) (note change from previous default 2) */
             }
             symbol->text[0] = '\0'; /* Disable HRT for compatibility with CODABLOCKF */
-#ifdef COMPLIANT_HEIGHTS
-            /* AIM ISS-X-24 Section 4.5.1 minimum row height 8 (for compatibility with CODABLOCKF, not specced for
-               CODE128) */
-            error_number = set_height(symbol, 8.0f, 10.0f, 0.0f, 0 /*no_errtxt*/);
-#else
-            (void) set_height(symbol, 0.0f, 5.0f, 0.0f, 1 /*no_errtxt*/);
-#endif
+            if (symbol->output_options & COMPLIANT_HEIGHT) {
+                /* AIM ISS-X-24 Section 4.5.1 minimum row height 8 (for compatibility with CODABLOCKF, not specced for
+                   CODE128) */
+                error_number = set_height(symbol, 8.0f, 10.0f, 0.0f, 0 /*no_errtxt*/);
+            } else {
+                (void) set_height(symbol, 0.0f, 5.0f, 0.0f, 1 /*no_errtxt*/);
+            }
         }
         return error_number;
     }
@@ -954,24 +935,27 @@ INTERNAL int codablock(struct zint_symbol *symbol, unsigned char source[], int l
 
     /* Paint the C128 patterns */
     for (r = 0; r < rows; r++) {
-        strcpy(dest, "");
-        for(c = 0; c < columns; c++) {
-            strcat(dest, C128Table[pOutput[r * columns + c]]);
+        const int rc = r * columns;
+        char *d = dest;
+        for (c = 0; c < columns - 1; c++, d += 6) {
+            memcpy(d, C128Table[pOutput[rc + c]], 6);
         }
-        expand(symbol, dest);
+        memcpy(d, "2331112", 7); /* Stop character (106, not in C128Table) */
+        d += 7;
+        expand(symbol, dest, d - dest);
     }
 
-#ifdef COMPLIANT_HEIGHTS
-    /* AIM ISS-X-24 Section 4.6.1 minimum row height; use 10 * rows as default for back-compatibility */
-    min_row_height = (float) (0.55 * useColumns + 3.0);
-    if (min_row_height < 8.0f) {
-        min_row_height = 8.0f;
+    if (symbol->output_options & COMPLIANT_HEIGHT) {
+        /* AIM ISS-X-24 Section 4.6.1 minimum row height; use 10 * rows as default */
+        float min_row_height = stripf(0.55f * useColumns + 3.0f);
+        if (min_row_height < 8.0f) {
+            min_row_height = 8.0f;
+        }
+        error_number = set_height(symbol, min_row_height, (min_row_height > 10.0f ? min_row_height : 10.0f) * rows,
+                                    0.0f, 0 /*no_errtxt*/);
+    } else {
+        (void) set_height(symbol, 0.0f, 10.0f * rows, 0.0f, 1 /*no_errtxt*/);
     }
-    error_number = set_height(symbol, min_row_height, (min_row_height > 10.0f ? min_row_height : 10.0f) * rows, 0.0f,
-                                0 /*no_errtxt*/);
-#else
-    (void) set_height(symbol, min_row_height, 10.0f * rows, 0.0f, 1 /*no_errtxt*/);
-#endif
 
     symbol->output_options |= BARCODE_BIND;
 
