@@ -30,38 +30,39 @@
  */
 /* vim: set ts=4 sw=4 et : */
 
+#include <assert.h>
 #include <math.h>
+#include <stdio.h>
 #include "common.h"
 #include "output.h"
 #include "font.h"
 
-#define SSET "0123456789ABCDEF"
+#define SSET_F  (IS_NUM_F | IS_UHX_F) /* SSET "0123456789ABCDEF" */
 
 /* Check colour options are good. Note: using raster.c error nos 651-654 */
-INTERNAL int output_check_colour_options(struct zint_symbol *symbol) {
-    int error_number;
+INTERNAL int out_check_colour_options(struct zint_symbol *symbol) {
+    int fg_len = (int) strlen(symbol->fgcolour);
+    int bg_len = (int) strlen(symbol->bgcolour);
 
-    if ((strlen(symbol->fgcolour) != 6) && (strlen(symbol->fgcolour) != 8)) {
-        strcpy(symbol->errtxt, "651: Malformed foreground colour target");
+    if ((fg_len != 6) && (fg_len != 8)) {
+        strcpy(symbol->errtxt, "651: Malformed foreground colour (6 or 8 characters only)");
         return ZINT_ERROR_INVALID_OPTION;
     }
-    if ((strlen(symbol->bgcolour) != 6) && (strlen(symbol->bgcolour) != 8)) {
-        strcpy(symbol->errtxt, "652: Malformed background colour target");
+    if ((bg_len != 6) && (bg_len != 8)) {
+        strcpy(symbol->errtxt, "652: Malformed background colour (6 or 8 characters only)");
         return ZINT_ERROR_INVALID_OPTION;
     }
 
-    to_upper((unsigned char *) symbol->fgcolour);
-    to_upper((unsigned char *) symbol->bgcolour);
+    to_upper((unsigned char *) symbol->fgcolour, fg_len);
+    to_upper((unsigned char *) symbol->bgcolour, bg_len);
 
-    error_number = is_sane(SSET, (unsigned char *) symbol->fgcolour, (int) strlen(symbol->fgcolour));
-    if (error_number == ZINT_ERROR_INVALID_DATA) {
-        strcpy(symbol->errtxt, "653: Malformed foreground colour target");
+    if (!is_sane(SSET_F, (unsigned char *) symbol->fgcolour, fg_len)) {
+        sprintf(symbol->errtxt, "653: Malformed foreground colour '%s' (hexadecimal only)", symbol->fgcolour);
         return ZINT_ERROR_INVALID_OPTION;
     }
 
-    error_number = is_sane(SSET, (unsigned char *) symbol->bgcolour, (int) strlen(symbol->bgcolour));
-    if (error_number == ZINT_ERROR_INVALID_DATA) {
-        strcpy(symbol->errtxt, "654: Malformed background colour target");
+    if (!is_sane(SSET_F, (unsigned char *) symbol->bgcolour, bg_len)) {
+        sprintf(symbol->errtxt, "654: Malformed background colour '%s' (hexadecimal only)", symbol->bgcolour);
         return ZINT_ERROR_INVALID_OPTION;
     }
 
@@ -69,7 +70,8 @@ INTERNAL int output_check_colour_options(struct zint_symbol *symbol) {
 }
 
 /* Return minimum quiet zones for each symbology */
-static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, float *top, float *bottom) {
+STATIC_UNLESS_ZINT_TEST int out_quiet_zones(const struct zint_symbol *symbol, const int hide_text,
+                            float *left, float *right, float *top, float *bottom) {
     int done = 0;
 
     *left = *right = *top = *bottom = 0.0f;
@@ -78,25 +80,33 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
     switch (symbol->symbology) {
         case BARCODE_CODE16K:
             /* BS EN 12323:2005 Section 4.5 (c) */
-            *left = 10.0f;
-            *right = 1.0f;
+            if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                *left = 10.0f;
+                *right = 1.0f;
+            }
             done = 1;
             break;
         case BARCODE_CODE49:
             /* ANSI/AIM BC6-2000 Section 2.4 */
-            *left = 10.0f;
-            *right = 1.0f;
+            if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                *left = 10.0f;
+                *right = 1.0f;
+            }
             done = 1;
             break;
         case BARCODE_CODABLOCKF:
         case BARCODE_HIBC_BLOCKF:
             /* AIM ISS-X-24 Section 4.6.1 */
-            *left = *right = 10.0f;
+            if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                *left = *right = 10.0f;
+            }
             done = 1;
             break;
         case BARCODE_ITF14:
             /* GS1 General Specifications 21.0.1 Section 5.3.2.2 */
-            *left = *right = 10.0f;
+            if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                *left = *right = 10.0f;
+            }
             done = 1;
             break;
         case BARCODE_EANX:
@@ -106,21 +116,33 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
             /* GS1 General Specifications 21.0.1 Section 5.2.3.4 */
             switch (ustrlen(symbol->text)) {
                 case 13: /* EAN-13 */
-                    *left = 11.0f;
-                    *right = 7.0f;
+                    if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                        *left = 11.0f;
+                        *right = 7.0f;
+                    } else if (!hide_text) {
+                        *left = 11.0f; /* Need for outside left digit */
+                    }
                     break;
                 case 16: /* EAN-13/ISBN + 2 digit addon */
                 case 19: /* EAN-13/ISBN + 5 digit addon */
-                    *left = 11.0f;
-                    *right = 5.0f;
+                    if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                        *left = 11.0f;
+                        *right = 5.0f;
+                    } else if (!hide_text) {
+                        *left = 11.0f; /* Need for outside left digit */
+                    }
                     break;
                 case 5: /* EAN-5 addon */
                 case 2: /* EAN-2 addon */
-                    *left = 7.0f;
-                    *right = 5.0f;
+                    if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                        *left = 7.0f;
+                        *right = 5.0f;
+                    }
                     break;
                 default: /* EAN-8 (+/- 2/5 digit addon) */
-                    *left = *right = 7.0f;
+                    if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                        *left = *right = 7.0f;
+                    }
                     break;
             }
             done = 1;
@@ -129,11 +151,18 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
         case BARCODE_UPCA_CHK:
         case BARCODE_UPCA_CC:
             /* GS1 General Specifications 21.0.1 Section 5.2.3.4 */
-            *left = 9.0f;
-            if (ustrlen(symbol->text) > 12) { /* UPC-A + addon */
-                *right = 5.0f;
-            } else {
-                *right = 9.0f;
+            if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                *left = 9.0f;
+                if (ustrlen(symbol->text) > 12) { /* UPC-A + addon */
+                    *right = 5.0f;
+                } else {
+                    *right = 9.0f;
+                }
+            } else if (!hide_text) {
+                *left = 9.0f; /* Need for outside left digit */
+                if (ustrlen(symbol->text) <= 12) { /* No addon */
+                    *right = 9.0f; /* Need for outside right digit */
+                }
             }
             done = 1;
             break;
@@ -141,11 +170,18 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
         case BARCODE_UPCE_CHK:
         case BARCODE_UPCE_CC:
             /* GS1 General Specifications 21.0.1 Section 5.2.3.4 */
-            *left = 9.0f;
-            if (ustrlen(symbol->text) > 8) { /* UPC-E + addon */
-                *right = 5.0f;
-            } else {
-                *right = 7.0f;
+            if (!(symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+                *left = 9.0f;
+                if (ustrlen(symbol->text) > 8) { /* UPC-E + addon */
+                    *right = 5.0f;
+                } else {
+                    *right = 7.0f;
+                }
+            } else if (!hide_text) {
+                *left = 9.0f; /* Need for outside left digit */
+                if (ustrlen(symbol->text) <= 8) { /* No addon */
+                    *right = 7.0f; /* Need for outside right digit */
+                }
             }
             done = 1;
             break;
@@ -155,14 +191,10 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
         return done;
     }
 
-    /* Only do others if flag set TODO: finish */
-#if 0
-    if (!(symbol->output_options & BARCODE_QUIET_ZONES)) {
-        return done;
+    /* Only do others if flag set */
+    if (!(symbol->output_options & BARCODE_QUIET_ZONES) || (symbol->output_options & BARCODE_NO_QUIET_ZONES)) {
+        return 0;
     }
-#else
-    return done;
-#endif
 
     switch (symbol->symbology) {
         case BARCODE_CODE11:
@@ -230,7 +262,7 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
             break;
 
         case BARCODE_FLAT:
-            /* TODO */
+            /* TODO: Find doc (application defined according to TEC-IT) */
             break;
 
         case BARCODE_DBAR_OMN: /* GS1 Databar Omnidirectional */
@@ -255,7 +287,10 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
 
         case BARCODE_TELEPEN:
         case BARCODE_TELEPEN_NUM:
-            /* TODO */
+            /* Appears to be ~10X from diagram in Telepen Barcode Symbology information and History */
+            /* TODO: Find better doc */
+            *left = *right = 10.0f;
+            done = 1;
             break;
 
         case BARCODE_POSTNET:
@@ -268,7 +303,9 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
             break;
 
         case BARCODE_MSI_PLESSEY:
-            /* TODO */
+            /* TODO Find doc (TEC-IT says 12X so use that for the moment) */
+            *left = *right = 12.0f;
+            done = 1;
             break;
 
         case BARCODE_FIM:
@@ -354,7 +391,9 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
             break;
 
         case BARCODE_KOREAPOST:
-            /* TODO */
+            /* TODO Find doc (TEC-IT uses 10X but says not exactly specified - do the same for the moment) */
+            *left = *right = 10.0f;
+            done = 1;
             break;
 
         case BARCODE_USPS_IMAIL:
@@ -366,7 +405,9 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
             break;
 
         case BARCODE_PLESSEY:
-            /* TODO */
+            /* TODO Find doc (see MSI_PLESSEY) */
+            *left = *right = 12.0f;
+            done = 1;
             break;
 
         case BARCODE_KIX:
@@ -408,7 +449,11 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
             break;
 
         case BARCODE_CODEONE:
-            /* TODO */
+            /* USS Code One AIM 1994 Section 2.2.4 No quiet zone required for Versions A to H */
+            if (symbol->option_2 == 9 || symbol->option_2 == 10) { /* Section 2.3.2 Versions S & T */
+                *left = *right = 1.0f;
+            }
+            done = 1;
             break;
 
         case BARCODE_GRIDMATRIX:
@@ -427,11 +472,12 @@ static int quiet_zones(struct zint_symbol *symbol, float *left, float *right, fl
 }
 
 /* Set left (x), top (y), right and bottom offsets for whitespace */
-INTERNAL void output_set_whitespace_offsets(struct zint_symbol *symbol, float *xoffset, float *yoffset,
-                float *roffset, float *boffset) {
+INTERNAL void out_set_whitespace_offsets(const struct zint_symbol *symbol, const int hide_text,
+                float *xoffset, float *yoffset, float *roffset, float *boffset, const float scaler,
+                int *xoffset_si, int *yoffset_si, int *roffset_si, int *boffset_si) {
     float qz_left, qz_right, qz_top, qz_bottom;
 
-    quiet_zones(symbol, &qz_left, &qz_right, &qz_top, &qz_bottom);
+    out_quiet_zones(symbol, hide_text, &qz_left, &qz_right, &qz_top, &qz_bottom);
 
     *xoffset = symbol->whitespace_width + qz_left;
     *roffset = symbol->whitespace_width + qz_right;
@@ -446,14 +492,29 @@ INTERNAL void output_set_whitespace_offsets(struct zint_symbol *symbol, float *x
         *yoffset += symbol->border_width;
         *boffset += symbol->border_width;
     }
+
+    if (scaler) {
+        if (xoffset_si) {
+            *xoffset_si = (int) (*xoffset * scaler);
+        }
+        if (yoffset_si) {
+            *yoffset_si = (int) (*yoffset * scaler);
+        }
+        if (roffset_si) {
+            *roffset_si = (int) (*roffset * scaler);
+        }
+        if (boffset_si) {
+            *boffset_si = (int) (*boffset * scaler);
+        }
+    }
 }
 
 /* Set composite offset and main width excluding addon (for start of addon calc) and addon text, returning
    UPC/EAN type */
-INTERNAL int output_process_upcean(struct zint_symbol *symbol, int *p_main_width, int *p_comp_offset,
+INTERNAL int out_process_upcean(const struct zint_symbol *symbol, int *p_main_width, int *p_comp_xoffset,
                 unsigned char addon[6], int *p_addon_gap) {
     int main_width; /* Width of main linear symbol, excluding addon */
-    int comp_offset; /* Whitespace offset (if any) of main linear symbol due to having composite */
+    int comp_xoffset; /* Whitespace offset (if any) of main linear symbol due to having composite */
     int upceanflag; /* UPC/EAN type flag */
     int i, j, latch;
     int text_length = (int) ustrlen(symbol->text);
@@ -480,10 +541,10 @@ INTERNAL int output_process_upcean(struct zint_symbol *symbol, int *p_main_width
     }
 
     /* Calculate composite offset */
-    comp_offset = 0;
+    comp_xoffset = 0;
     if (is_composite(symbol->symbology)) {
-        while (!(module_is_set(symbol, symbol->rows - 1, comp_offset))) {
-            comp_offset++;
+        while (!(module_is_set(symbol, symbol->rows - 1, comp_xoffset))) {
+            comp_xoffset++;
         }
     }
 
@@ -495,7 +556,7 @@ INTERNAL int output_process_upcean(struct zint_symbol *symbol, int *p_main_width
             case 13: /* EAN-13 */
             case 16: /* EAN-13 + EAN-2 */
             case 19: /* EAN-13 + EAN-5 */
-                main_width = 95 + comp_offset; /* EAN-13 main symbol 95 modules wide */
+                main_width = 95 + comp_xoffset; /* EAN-13 main symbol 95 modules wide */
                 upceanflag = 13;
                 break;
             case 2:
@@ -507,69 +568,93 @@ INTERNAL int output_process_upcean(struct zint_symbol *symbol, int *p_main_width
                 upceanflag = 5;
                 break;
             default:
-                main_width = 68 + comp_offset; /* EAN-8 main symbol 68 modules wide */
+                main_width = 68 + comp_xoffset; /* EAN-8 main symbol 68 modules wide */
                 upceanflag = 8;
                 break;
         }
     } else if ((symbol->symbology == BARCODE_UPCA) || (symbol->symbology == BARCODE_UPCA_CHK)
             || (symbol->symbology == BARCODE_UPCA_CC)) {
-        main_width = 95 + comp_offset; /* UPC-A main symbol 95 modules wide */
+        main_width = 95 + comp_xoffset; /* UPC-A main symbol 95 modules wide */
         upceanflag = 12;
     } else if ((symbol->symbology == BARCODE_UPCE) || (symbol->symbology == BARCODE_UPCE_CHK)
             || (symbol->symbology == BARCODE_UPCE_CC)) {
-        main_width = 51 + comp_offset; /* UPC-E main symbol 51 modules wide */
+        main_width = 51 + comp_xoffset; /* UPC-E main symbol 51 modules wide */
         upceanflag = 6;
     }
 
-    *p_comp_offset = comp_offset;
+    *p_comp_xoffset = comp_xoffset;
     *p_main_width = main_width;
 
     return upceanflag;
 }
 
 /* Calculate large bar height i.e. linear bars with zero row height that respond to the symbol height.
-   If scaler `si` non-zero (raster), then large_bar_height if non-zero or else row heights will be rounded to nearest
-   pixel and symbol height adjusted */
-INTERNAL float output_large_bar_height(struct zint_symbol *symbol, int si) {
+   If scaler `si` non-zero (raster), then large_bar_height if non-zero or else row heights will be rounded
+   to nearest pixel and symbol height adjusted */
+INTERNAL float out_large_bar_height(struct zint_symbol *symbol, int si, int *row_heights_si, int *symbol_height_si) {
     float fixed_height = 0.0f;
     int zero_count = 0;
     int round_rows = 0;
     int i;
-    float large_bar_height;
+    float large_bar_height = 0.0f; /* Not used if zero_count zero */
 
-    for (i = 0; i < symbol->rows; i++) {
-        if (symbol->row_height[i]) {
-            fixed_height += symbol->row_height[i];
-            if (!round_rows && si && !isfintf(symbol->row_height[i] * si)) {
-                round_rows = 1;
-            }
-        } else {
-            zero_count++;
-        }
-    }
-
-    if (zero_count) {
-        large_bar_height = (symbol->height - fixed_height) / zero_count;
-        if (large_bar_height <= 0.0f) { /* Shouldn't happen but protect against memory access violations */
-            large_bar_height = 0.01f; /* Token positive value */
-            symbol->height = large_bar_height * zero_count + fixed_height;
-        }
-        if (si && !isfintf(large_bar_height * si)) {
-            large_bar_height = roundf(large_bar_height * si) / si;
-            symbol->height = large_bar_height * zero_count + fixed_height;
-        }
-        /* Note should never happen that have both zero_count and round_rows */
-    } else {
-        large_bar_height = 0.0f; /* Not used if zero_count zero */
-        if (round_rows) {
-            fixed_height = 0.0f;
-            for (i = 0; i < symbol->rows; i++) {
-                if (!isfintf(symbol->row_height[i] * si)) {
-                    symbol->row_height[i] = roundf(symbol->row_height[i] * si) / si;
-                }
+    if (si) {
+        for (i = 0; i < symbol->rows; i++) {
+            if (symbol->row_height[i]) {
                 fixed_height += symbol->row_height[i];
+                if (!round_rows && !isfintf(symbol->row_height[i] * si)) {
+                    round_rows = 1;
+                }
+            } else {
+                zero_count++;
             }
-            symbol->height = fixed_height;
+        }
+
+        if (zero_count) {
+            large_bar_height = stripf((symbol->height - fixed_height) / zero_count);
+            assert(large_bar_height >= 0.5f); /* Min row height as set by `set_height()` */
+            if (!isfintf(large_bar_height * si)) {
+                large_bar_height = stripf(roundf(large_bar_height * si) / si);
+            }
+            symbol->height = stripf(large_bar_height * zero_count + fixed_height);
+            /* Note should never happen that have both zero_count and round_rows */
+        } else {
+            if (round_rows) {
+                float total_height = 0.0f;
+                for (i = 0; i < symbol->rows; i++) {
+                    if (!isfintf(symbol->row_height[i] * si)) {
+                        symbol->row_height[i] = roundf(symbol->row_height[i] * si) / si;
+                    }
+                    total_height += symbol->row_height[i];
+                }
+                symbol->height = stripf(total_height);
+            }
+        }
+
+        if (row_heights_si) {
+            assert(symbol_height_si);
+            *symbol_height_si = 0;
+            for (i = 0; i < symbol->rows; i++) {
+                if (symbol->row_height[i]) {
+                    row_heights_si[i] = (int) roundf(symbol->row_height[i] * si);
+                } else {
+                    row_heights_si[i] = (int) roundf(large_bar_height * si);
+                }
+                *symbol_height_si += row_heights_si[i];
+            }
+        }
+    } else {
+        for (i = 0; i < symbol->rows; i++) {
+            if (symbol->row_height[i]) {
+                fixed_height += symbol->row_height[i];
+            } else {
+                zero_count++;
+            }
+        }
+        if (zero_count) {
+            large_bar_height = stripf((symbol->height - fixed_height) / zero_count);
+            assert(large_bar_height >= 0.5f); /* Min row height as set by `set_height()` */
+            symbol->height = stripf(large_bar_height * zero_count + fixed_height);
         }
     }
 
@@ -577,7 +662,7 @@ INTERNAL float output_large_bar_height(struct zint_symbol *symbol, int si) {
 }
 
 /* Split UPC/EAN add-on text into various constituents */
-INTERNAL void output_upcean_split_text(int upceanflag, unsigned char text[],
+INTERNAL void out_upcean_split_text(int upceanflag, unsigned char text[],
                 unsigned char textpart1[5], unsigned char textpart2[7], unsigned char textpart3[7],
                 unsigned char textpart4[2]) {
     int i;
