@@ -16,26 +16,35 @@ function run_bwipp_test() {
 run_bwipp_test "test_2of5" "encode"
 run_bwipp_test "test_auspost" "encode"
 run_bwipp_test "test_aztec" "encode"
+run_bwipp_test "test_aztec" "encode_segs"
 run_bwipp_test "test_channel" "encode"
 run_bwipp_test "test_codablock" "encode"
 run_bwipp_test "test_code" "encode"
 run_bwipp_test "test_code1" "encode"
+run_bwipp_test "test_code1" "encode_segs"
 run_bwipp_test "test_code128" "encode"
 run_bwipp_test "test_code16k" "encode"
 run_bwipp_test "test_code49" "encode"
 run_bwipp_test "test_composite"
-run_bwipp_test "test_dmatrix"
+run_bwipp_test "test_dmatrix" "input"
+run_bwipp_test "test_dmatrix" "encode"
+run_bwipp_test "test_dmatrix" "encode_segs"
+run_bwipp_test "test_dotcode" "input"
 run_bwipp_test "test_dotcode" "encode"
+run_bwipp_test "test_dotcode" "encode_segs"
 run_bwipp_test "test_gs1" "gs1_reduce"
 run_bwipp_test "test_imail" "encode"
+run_bwipp_test "test_maxicode" "input"
 run_bwipp_test "test_maxicode" "encode"
+run_bwipp_test "test_maxicode" "encode_segs"
 run_bwipp_test "test_medical" "encode"
 run_bwipp_test "test_pdf417" "encode"
+run_bwipp_test "test_pdf417" "encode_segs"
 run_bwipp_test "test_plessey" "encode"
 run_bwipp_test "test_postal" "encode"
-run_bwipp_test "test_qr" "microqr_encode"
-run_bwipp_test "test_qr" "rmqr_encode"
+run_bwipp_test "test_qr"
 run_bwipp_test "test_rss"
 run_bwipp_test "test_telepen" "encode"
+run_bwipp_test "test_upcean" "upce_input"
 run_bwipp_test "test_upcean" "encode"
 run_bwipp_test "test_ultra" "encode"
